#ifndef _SHOT_H_
#define _SHOT_H_

#include <iostream>
#include "chamber.h"

using namespace std;

// generic ensemble of spheres in an external (homogeneous) gravitational field
class Shot
{
// shot properties
    int n_;
    double diam_, diam2_; // diameter, diameter squared
    double mass_;
    double v_scale_; // inverse of threshold velocity

// state vector
    double **r_, **v_;

// collision data
    double *tnext_; // time to next collision for each particle
    int *partner_; // its collision partner
    int *update_; // collision update flag
    int chosen_; // chosen particle
    double best_time_; // best time yet (including chamber impacts)
    int chamber_impact_; // 0 for intrashot impacts, 1 otherwise

// collision counter
    int collision_;

    double coef(double velocity); // restitution coefficient

public:
    Shot(const char* filename);
    ~Shot();

    int n() const;
    int collision() const;
    double diam() const;
    double tNext() const;

    void distribute(Chamber& chamber); // random distribution of the spheres

    void advance(double time, double gravity);
    void detect();
    void correct(Chamber& chamber, double gravity);
    void collide(Chamber& chamber);

    void test(const Chamber& chamber);
    void dumpState(const char* filename);
};

inline int Shot::n() const {return n_;}
inline int Shot::collision() const {return collision_;}
inline double Shot::diam() const {return diam_;}
inline double Shot::tNext() const {return best_time_;}

#endif
